/****************************************************************
 *  zx97copy.c
 *		FPGA-Konfiguration um Bootkopierer erweitern
 *		============================================
 *
 *  03.02.99	Bodo Wenzel	V1.0, quick-and-dirty
 *  23.02.99	Bodo Wenzel	V1.1, umbenannt
 ****************************************************************/

/* Vor den eigentlichen FPGA-Daten knnen weitere Bytes kommen,
   die aber nicht das Bitmuster 0100 enthalten drfen (auch ber
   Bytegrenzen (z. B. 00xxxxxx xxxxxx01). Die Anzahl Bits in den
   FPGA-Daten mu dann entsprechend angepat werden.
*/

#include <stdio.h>

#define  FPGA_SIZE   (0xF0E)

#define  COPY_START  (0xF18)	/* mindestens um JUMP_SIZE	*/
				/* grer als FPGA_SIZE und	*/
				/* keine einzelnen 1-Bits!	*/

#define  BUF_SIZE    (COPY_START+16)

typedef  unsigned char  uchar;
typedef  unsigned int   uint;

#define  JUMP_SIZE  (3)		/* sonst geht der Test nicht...	*/
uchar  jump_code[JUMP_SIZE]={
  0xC3,			/*	jp	COPY_START		*/
  (uchar)(COPY_START&0xFF),
  (uchar)(COPY_START>>8)};
#if COPY_START < FPGA_SIZE+JUMP_SIZE
#error Bitte COPY_START anpassen!
#endif

#define  COPY_SIZE  (16)
uchar  copy_code[COPY_SIZE]={
  0x21,			/*	ld	hl,BUF_SIZE		*/
  (uchar)(BUF_SIZE&0xFF),
  (uchar)(BUF_SIZE>>8),
  0x11,0x00,0x80,	/*	ld	de,8000H		*/
  0x01,			/*	ld	bc,2000H-BUF_SIZE	*/
  (uchar)((0x2000-BUF_SIZE)&0xFF),
  (uchar)((0x2000-BUF_SIZE)>>8),
  0xED,0xB0,		/*	ldir				*/
  0x3E,0x88,		/*	ld	a,10001000B		*/
  0x32,0x07,0x00};	/*	ld	(POKE7),a		*/
#if BUF_SIZE < COPY_START+COPY_SIZE
#error Bitte BUF_SIZE anpassen!
#endif

uchar  buffer[BUF_SIZE];

int main(int argc,char *argv[]) {
/*-----------------------------*/
FILE  *f;
int   i;
long  l1,l2;

if (argc!=3) {
  puts("usage: zx97copy fpga-file destination");
  return(1);
  }

f=fopen(argv[1],"rb");
if (f==NULL) {
  printf("can't open fpga-file!");
  return(2);
  }
fseek(f,0,SEEK_END);
if (ftell(f)!=FPGA_SIZE) {
  printf("wrong size of fpga-file!");
  fclose(f);
  return(2);
  }
fseek(f,0,SEEK_SET);
if (fread(buffer+JUMP_SIZE,
          sizeof(uchar),FPGA_SIZE,f)!=FPGA_SIZE) {
  printf("read error on fpga-file!");
  fclose(f);
  return(2);
  }
fclose(f);

for (i=0; i<JUMP_SIZE; i++)
  buffer[0+i]=jump_code[i];
l1=((long)buffer[JUMP_SIZE+4]<<24)+
   ((long)buffer[JUMP_SIZE+3]<<16)+
   ((long)buffer[JUMP_SIZE+2]<<8)+
   ((long)buffer[JUMP_SIZE+1]<<0);
l2=0;
for (i=0; i<24; i++) {
  l2<<=1;
  if (l1&0x10)
    l2++;
  l1>>=1;
  }
l2+=JUMP_SIZE*8;
l1=0xF;
for (i=0; i<24; i++) {
  l1<<=1;
  if (l2&0x1)
    l1++;
  l2>>=1;
  }
l1<<=4;
l1|=0x4;
for (i=0; i<COPY_SIZE; i++)
  buffer[COPY_START+i]=copy_code[i];
buffer[JUMP_SIZE+4]=(uchar)(l1>>24);
buffer[JUMP_SIZE+3]=(uchar)(l1>>16);
buffer[JUMP_SIZE+2]=(uchar)(l1>>8);
buffer[JUMP_SIZE+1]=(uchar)(l1>>0);

f=fopen(argv[2],"wb");
if (f==NULL) {
  puts("can't open destination!");
  return(3);
  }
fwrite(buffer,sizeof(uchar),BUF_SIZE,f);
if (ferror(f)) {
  printf("write error on destination!");
  fclose(f);
  return(3);
  }
fclose(f);

return(0);
}

/* ende */
