;****************************************************************
;*  zx97dumm.asm
;*			Bootmanager fr ZX97 (Dummie-Version)
;*			=====================================
;*
;*  01.03.99	Bodo Wenzel	V1.0, Erstellung
;****************************************************************

	title	"ZX97 Manager (Dummie-Version) V1.0"

;= Startadressen (laut zx97boot.c) ==============================

RES_START	equ	0F77H
NMI_START	equ	0F7CH

;= I/O ==========================================================

POKE7		equ	0007H	;Adresse fr FPGA-Modus

NMI_OFF		equ	0FDH	;per OUT
NMI_ON		equ	0FEH	;per OUT
KEYB		equ	0FEH	;per IN
HSYNC_ON	equ	0FFH	;per OUT

;= Konstanten ===================================================

MODE_DEFAULT	equ	11111011B
			;7	RESET
			;6	0/1 = RAM/ROM in 0-8K
			;5	PAGE1
			;4	PAGE0 Seite fr 0-8K
			;3	0/1 = invers/normal
			;2	0/1 = HRG/Text in 8-16K
			;1	RAM1
			;0	RAM0 Modus fr RAM

;	ACHTUNG! Da die Bilderzeugung extrem vom durchlaufenen
;	Maschinencode abhngt, knnen diese Konstanten nicht
;	ohne weiteres gendert werden! Sie dienen nur der
;	besseren Lesbarkeit.

LINES		equ	240	;Anzahl Zeilen
COLUMNS		equ	320/8	;Anzahl Spalten [Bytes]
OFFS		equ	512/8	;nchstgrere 2er-Potenz
LINES_TOP	equ	3	;Leerzeilen oben
MARGIN_TOP	equ	8*LINES_TOP-1-1
MARGIN_BOTTOM	equ	312-(1+1+6+MARGIN_TOP+1+1+LINES)
				;sorry, keine 60 Hz mglich

N_SQUIXX	equ	8	;Anzahl Linien

W_ROWS		equ	16	;Gre des Textfensters
W_COLS		equ	22

		if	W_COLS > OFFS-COLUMNS
		error	"Zuviele Textspalten!"
		endif
		if	8*W_ROWS > LINES
		error	"Zuviele Textzeilen!"
		endif

XOFFS		equ	(COLUMNS-W_COLS)/2
YOFFS		equ	(LINES-8*W_ROWS)/2

FONT		equ	1E00H	;Startadresse Zeichensatz

;= Codeanfang (nach FPGA-Konfiguration) =========================

	org	RES_START
	jp	reset

;= NMI fr Zeilenzhlung ========================================

	org	NMI_START
nmi:
	ex	af,af'
	dec	a
	jr	z,brkcal	;alle Leerzeilen bearbeitet ?
	ex	af,af'
	ret

brkcal:
	ex	af,af'
	push	af
	push	bc
	push	de
	push	hl		;Register retten

	halt			;synchronisieren mit HSync
	out	(NMI_OFF),a

	jp	(ix)		;abwechselnd "sync" und "show"
;----------------------------------------------------------------
rstcal:
	pop	ix		;Rcksprung merken

	ex	af,af'
	out	(NMI_ON),a	;NMIs zur Zeilenzhlung

	pop	hl
	pop	de
	pop	bc
	pop	af		;Register holen
	ret

;= Start (Reset) ================================================

reset:
	out	(NMI_OFF),a
	di			;schnell stoppen
;----------------------------------------------------------------
	ld	hl,line
	ld	de,dummy
	ld	bc,LINE_LEN
	ldir			;Dummycode ins RAM kopieren

	ld	sp,line_stack+(1+LINES*2)*2
	ld	hl,show_end
	push	hl
	ld	a,LINES
	ld	bc,dummy+8000H
	ld	hl,video_mem+LINES*OFFS
	ld	de,-OFFS
ls_loop:
	add	hl,de
	push	hl
	push	bc
	dec	a
	jr	nz,ls_loop	;Stack fr Bildausgabe setzen

	ld	sp,stack	;dies ist der "echte" Stack

	ld	ix,sync
	out	(NMI_ON),a	;Bildausgabe starten
;----------------------------------------------------------------
	ld	hl,packed
	ld	de,video_mem+LINES*(OFFS-COLUMNS)
	ld	b,1		;Auspacken vorbereiten
	push	bc
unp_loop:
	pop	bc

	bit	7,d
	jr	nz,expand	;fertig mit Auspacken?

	djnz	unp_goon	;noch Bits im Prefixbyte?

	ld	c,(hl)		;nchstes Prefixbyte
	inc	hl
	ld	b,8		;Bitzhler zurcksetzen

unp_goon:
	rl	c
	push	bc
	jr	c,unp_repl	;Wiederholung?

	ldi			;Byte direkt kopieren
	jr	unp_loop

unp_repl:
	ld	c,(hl)
	inc	hl
	ld	a,(hl)
	and	0fh
	ld	b,a
	rld
	inc	hl		;Anzahl und Offset holen

	push	hl
	ld	hl,-3
	sbc	hl,bc
	add	hl,de
	add	a,3
	ld	c,a
	ld	b,0
	ldir
	pop	hl		;Wiederholung kopieren
	jr	unp_loop

expand:
	ld	hl,video_mem+LINES*(OFFS-COLUMNS)
	ld	de,video_mem
	ld	a,LINES		;Umspeicherung vorbereiten
exp_loop:
	ld	bc,COLUMNS
	ldir
	ex	de,hl
	ld	bc,OFFS-COLUMNS
	add	hl,bc
	ex	de,hl
	dec	a
	jr	nz,exp_loop	;noch nicht alle Zeilen?
;----------------------------------------------------------------
	ld	a,MODE_DEFAULT
	ld	(mode_act),a	;Standard-Modus vorgeben

	ld	hl,video_mem+YOFFS*OFFS+COLUMNS
	ld	(scr_base),hl	;erstmal im Hintergrund

	ld	hl,cls_text
	ld	b,W_ROWS
cls_loop:
	push	bc
	call	puts
	pop	bc
	djnz	cls_loop	;noch nicht ganzer Text?

	ld	hl,video_mem+YOFFS*OFFS+XOFFS
	ld	(scr_base),hl	;ab jetzt im Vordergrund

	jp	hrg_start	;auf geht's!
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
cls_text:
	dw	0*8*OFFS
	db	07H,03H,03H,03H,03H,03H,03H,03H,03H,03H,03H
	db	03H,03H,03H,03H,03H,03H,03H,03H,03H,03H,0C4H
	dw	1*8*OFFS
	db	05H,08H,08H,08H,08H,08H,08H,08H,08H,08H,08H
	db	08H,08H,08H,08H,08H,08H,08H,08H,08H,08H,0C5H
	dw	2*8*OFFS
	db	05H,08H,00H,2AH,2EH,33H,38H,39H,2AH,31H,31H
	db	3AH,33H,2CH,2AH,33H,00H,00H,00H,00H,08H,0C5H
	dw	3*8*OFFS
	db	05H,08H,08H,08H,08H,08H,08H,08H,08H,08H,08H
	db	08H,08H,08H,08H,08H,08H,08H,08H,08H,08H,0C5H
	dw	4*8*OFFS
	db	05H,00H,00H,00H,00H,00H,00H,00H,00H,00H,00H
	db	00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,0C5H
	dw	5*8*OFFS
	db	05H,0B8H,3EH,38H,39H,2AH,32H,0EH,00H,37H,0FH
	db	32H,00H,00H,00H,00H,00H,00H,00H,00H,00H,0C5H
	dw	6*8*OFFS
	db	05H,00H,00H,00H,00H,00H,00H,00H,00H,0B3H,37H
	db	1BH,0FH,00H,14H,00H,0FH,1CH,1CH,1CH,2DH,0C5H
	dw	7*8*OFFS
	db	05H,00H,00H,00H,00H,00H,00H,00H,00H,00H,00H
	db	00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,0C5H
	dw	8*8*OFFS
	db	05H,0BBH,2EH,29H,2AH,34H,0EH,00H,00H,00H,00H
	db	00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,0C5H
	dw	9*8*OFFS
	db	05H,00H,00H,00H,00H,00H,00H,00H,00H,0ADH,37H
	db	2CH,00H,26H,27H,00H,0FH,1CH,1CH,1CH,2DH,0C5H
	dw	10*8*OFFS
	db	05H,00H,00H,00H,00H,00H,00H,00H,00H,00H,00H
	db	00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,0C5H
	dw	11*8*OFFS
	db	05H,0B7H,26H,32H,0EH,00H,00H,00H,00H,0FH,1CH
	DB	1CH,1CH,2DH,16H,0FH,2BH,2BH,2BH,2DH,00H,0C5H
	dw	12*8*OFFS
	db	05H,00H,00H,00H,00H,00H,00H,00H,00H,00H,00H
	db	00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,0C5H
	dw	13*8*OFFS
	db	05H,00H,00H,00H,00H,00H,00H,00H,00H,00H,00H
	db	00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,0C5H
	dw	14*8*OFFS
	db	05H,0B4H,30H,00H,00H,00H,00H,00H,00H,00H,00H
	db	00H,00H,00H,00H,00H,00H,00H,00H,00H,00H,0C5H
	dw	15*8*OFFS
	db	82H,83H,83H,83H,83H,83H,83H,83H,83H,83H,83H
	db	83H,83H,83H,83H,83H,83H,83H,83H,83H,83H,0C1H
;----------------------------------------------------------------
hrg_start:
	ld	b,3*2*N_SQUIXX
	ld	hl,squixx
	xor	a
hrg_sloop:
	ld	(hl),a
	inc	hl
	djnz	hrg_sloop	;alle Koordinaten auf 0 setzen

	ld	hl,speed
	ld	(hl),7
	inc	hl
	ld	(hl),3
	inc	hl
	ld	(hl),11
	inc	hl
	ld	(hl),7		;ein paar empirische Werte
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
hrg_loop:
	ld	bc,(squixx)
	ld	a,(squixx+2)
	ld	e,a
	ld	d,0
	exx
	ld	bc,(squixx+3)
	ld	a,(squixx+3+2)
	ld	e,a
	ld	d,0
	call	xor_line	;letzte Strecke lschen

	ld	hl,squixx+3*2
	ld	de,squixx
	ld	bc,3*2*(N_SQUIXX-1)
	ldir			;Strecken im Puffer schieben

	ld	hl,squixx+3*2*(N_SQUIXX-1)
	ld	de,speed
	ld	b,2		;Vorbereitung fr zwei Punkte
hrg_step:
	push	bc

	ld	a,(hl)
	inc	hl
	push	hl
	ld	h,(hl)
	ld	l,a
	ld	a,(de)
	ld	c,a
	rlca
	jr	c,hsx_minus	;X-Schritt negativ?

	ld	b,0
	add	hl,bc
	push	hl
	ld	bc,-320
	add	hl,bc
	pop	bc		;positiver X-Schritt
	jr	nc,hsx_store
	ld	bc,319
	ld	a,(de)
	neg
	ld	(de),a
	jr	hsx_store	;begrenzen

hsx_minus:
	ld	b,-1
	add	hl,bc
	ld	b,h
	ld	c,l		;negativer Y-Schritt
	jr	c,hsx_store
	ld	bc,0
	ld	a,(de)
	neg
	ld	(de),a		;begrenzen

hsx_store:
	pop	hl
	ld	(hl),b
	dec	hl
	ld	(hl),c		;neue X-Koordinate merken
	inc	hl
	inc	hl
	inc	de

	ld	a,(de)
	ld	c,a
	rlca
	jr	c,hsy_minus	;Y-Schritt negativ?

	ld	a,(hl)
	add	a,c		;positiver Y-Schritt
	jr	c,hsy_over
	cp	240
	jr	c,hsy_store
hsy_over:
	ld	a,(de)
	neg
	ld	(de),a
	ld	a,239
	jr	hsy_store	;begrenzen

hsy_minus:
	ld	a,(hl)
	add	a,c		;negativer Y-Schritt
	jr	c,hsy_store
	ld	a,(de)
	neg
	ld	(de),a
	xor	a		;begrenzen

hsy_store:
	ld	(hl),a		;neue Y-Koordinate merken
	inc	hl
	inc	de

	pop	bc
	djnz	hrg_step	;noch nicht beide Punkte?

	ld	bc,(squixx+3*2*(N_SQUIXX-1))
	ld	a,(squixx+3*2*(N_SQUIXX-1)+2)
	ld	e,a
	ld	d,0
	exx
	ld	bc,(squixx+3*2*(N_SQUIXX-1)+3)
	ld	a,(squixx+3*2*(N_SQUIXX-1)+3+2)
	ld	e,a
	ld	d,0
	call	xor_line	;neue Strecke zeichnen
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	ld	a,(keys+8)
	inc	a
	jp	z,hrg_loop	;keine Taste gedrckt?
;----------------------------------------------------------------
	call	swap_pic	;Text einblenden

	ld	iy,mode_act	;Zeiger auf Einstellungen
txt_loop:
	ld	a,34H
	bit	6,(iy+0)
	jr	nz,$+4
	ld	a,26H
	ld	hl,10 + 5*8*OFFS
	ld	(scr_offs),hl
	call	putchar		;ROM/RAM ausgeben

	ld	a,(iy+0)
	and	30H
	rrca
	rrca
	rrca
	push	af
	rrca
	add	a,1CH
	ld	hl,12 + 6*8*OFFS
	ld	(scr_offs),hl
	call	putchar
	pop	af
	add	a,1CH
	ld	hl,16 + 6*8*OFFS
	ld	(scr_offs),hl
	call	putchar		;Blocknummer ausgeben

	ld	hl,txt_normal
	bit	3,(iy+0)
	jr	nz,$+5
	ld	hl,txt_invers
	call	puts		;Videosignal ausgeben

	ld	a,1EH
	bit	2,(iy+0)
	jr	z,$+4
	ld	a,20H
	ld	hl,16 + 9*8*OFFS
	ld	(scr_offs),hl
	call	putchar		;HRG-Startadresse ausgeben

	ld	a,1EH
	bit	1,(iy+0)
	jr	nz,$+8
	bit	0,(iy+0)
	jr	nz,$+4
	ld	a,20H
	ld	hl,9 + 11*8*OFFS
	ld	(scr_offs),hl
	call	putchar
	ld	a,2BH
	bit	0,(iy+0)
	jr	nz,$+4
	ld	a,27H
	bit	1,(iy+0)
	jr	nz,$+4
	ld	a,25H
	ld	hl,15 + 11*8*OFFS
	ld	(scr_offs),hl
	call	putchar		;RAM-Modus ausgeben
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	ld	hl,keys+8
txt_rloop:
	ld	a,(hl)
	inc	a
	jr	nz,txt_rloop	;noch Taste/n gedrckt?

	ld	b,5*50
txt_wait:
	ld	(hl),a		;Testwert setzen
txt_wloop:
	ld	a,(hl)
	and	a
	jr	z,txt_wloop	;noch kein VSync?

	inc	a
	jr	nz,txt_pressed	;Taste gedrckt?

	djnz	txt_wait	;noch kein Timeout?

	call	swap_pic	;Text ausblenden

	jp	hrg_loop	;zurck zur Grafik
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
txt_pressed:
	ld	a,80H-1
txt_ploop1:
	inc	a
	rrc	(hl)
	jr	c,txt_ploop1	;Spalte bestimmen

	ld	b,10H
txt_ploop2:
	sub	b
	dec	hl
	inc	(hl)
	jr	z,txt_ploop2	;Zeile bestimmen

	ld	bc,4040H
	cp	11H
	jr	z,txt_incr	;'S'?
	ld	bc,1030H
	cp	73H
	jr	z,txt_incr	;'N'?
	ld	bc,0808H
	cp	04H
	jr	z,txt_incr	;'V'?
	ld	bc,0404H
	cp	64H
	jr	z,txt_incr	;'H'?
	ld	bc,0103H
	cp	23H
	jr	z,txt_incr	;'R'?
	cp	51H
	jr	z,txt_ok	;'O'?
	jp	txt_loop
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
txt_incr:
	ld	a,(iy+0)
	ld	d,a
	add	a,b
	and	c
	ld	e,a
	ld	a,c
	cpl
	and	d
	or	e
	ld	(iy+0),a	;Einstellung weiterschalten
	jp	txt_loop
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
txt_ok:
	halt
	out	(NMI_OFF),a	;Bildausgabe stoppen

	ld	a,(iy+0)
	ld	(POKE7),a	;neues System starten...
				;(kommt bestimmt nicht weiter!)
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
txt_normal:
	dw	9 + 8*8*OFFS
	db	33H,34H,37H,32H,26H,71H
txt_invers:
	dw	9 + 8*8*OFFS
	db	2EH,33H,3BH,2AH,37H,78H

;= Grafikschirm =================================================

xor_line:			;bc=x1 de=y1 bc'=x2 de'=y2
	push	bc
	exx
	pop	hl
	and	a
	sbc	hl,bc
	jr	nc,xl_noswap
	exx			;x1<=x2 sicherstellen
xl_noswap:

	ld	a,c
	and	07H
	ld	l,a
	inc	l
	ld	a,0
	scf
xl_pixel:
	rra
	dec	l
	jr	nz,xl_pixel
	push	af		;Pixel im Byte bestimmen

	push	bc
	exx
	ld	h,b
	ld	l,c
	pop	bc
	sbc	hl,bc
	exx			;Anzahl X-Pixel berechnen

				;(s1)=m bc=x1 de=y1 de'=y2 hl'=dx
	push	de
	ex	de,hl
	add	hl,hl
	add	hl,hl
	add	hl,hl
	add	hl,hl
	add	hl,hl
	add	hl,hl
	ex	de,hl
	ld	h,b
	ld	l,c
	srl	h
	rr	l
	srl	h
	rr	l
	srl	h
	rr	l
	add	hl,de
	ld	de,video_mem
	add	hl,de
	pop	de
	push	hl		;Zeiger in Bildspeicher

	exx
	push	de
	exx
	pop	hl
	sbc	hl,de		;Anzahl Y-Pixel berechnen
	ld	de,+OFFS
	jr	nc,xl_down	;abwrts?
	ex	de,hl
	ld	hl,0
	and	a
	sbc	hl,de
	ld	de,-OFFS	;sonst Vorzeichen umdrehen
xl_down:
	push	hl
	exx
	pop	bc

				;(s2)=m (s1)=p de'=dp bc=dy hl=dx
	push	hl
	and	a
	sbc	hl,bc
	pop	hl
	jr	c,xl_steep	;Gerade steil?

xl_flat:
	push	hl
	exx
	pop	bc
	inc	bc		;Schrittzahl aus X-Pixeln
	exx

	sla	c
	rl	b
	ex	de,hl
	ld	hl,0
	sbc	hl,de
	ld	d,h
	ld	e,l
	add	hl,bc
	sla	e
	rl	d
	exx			;Vorbereitungen Bresenham
	pop	hl
				;(s1)=m bc=n de=dp hl=p
				;bc'=dy de'=-dx hl'=e
xlf_loop:
	pop	af
	push	af
	xor	(hl)
	ld	(hl),a		;Pixel umkippen

	exx
	bit	7,h
	jr	nz,xlf_br
	add	hl,de
	exx
	add	hl,de
	exx			;Bresenham lt gren
xlf_br:

	pop	af
	rrc	a
	push	af
	jr	nc,xlf_step
	exx
	inc	hl
	exx			;nchster X-Schritt
xlf_step:

	add	hl,bc
	exx
	dec	bc
	ld	a,b
	or	c
	jr	nz,xlf_loop	;noch nicht alle Schritte?

	pop	af
	ret			;fertig!

xl_steep:
	push	bc
	exx
	pop	bc
	inc	bc		;Schrittzahl aus Y-Pixeln
	exx

	add	hl,hl
	ex	de,hl
	ld	hl,0
	sbc	hl,bc
	ld	b,h
	ld	c,l
	add	hl,de
	sla	c
	rl	b
	exx			;Vorbereitungen Bresenham
	pop	hl
				;(s1)=m bc=n de=dp hl=p
				;bc'=-dy de'=dx hl'=e
xls_loop:
	pop	af
	push	af
	xor	(hl)
	ld	(hl),a		;Pixel umkippen

	exx
	bit	7,h
	jr	nz,xls_br
	pop	af
	rrc	a
	push	af
	jr	nc,xls_step
	exx
	inc	hl
	exx
xls_step:
	add	hl,bc		;Bresenham lt gren
xls_br:

	add	hl,de
	exx
	add	hl,de		;nchster Y-Schritt

	dec	bc
	ld	a,b
	or	c
	jr	nz,xls_loop	;noch nicht alle Schritte?

	pop	af
	ret			;fertig!

;= Textschirm ===================================================

swap_pic:
	ld	hl,video_mem+YOFFS*OFFS+XOFFS
	ld	de,video_mem+YOFFS*OFFS+COLUMNS

	ld	b,8*W_ROWS	;Anzahl Pixelzeilen
sp_loop1:
	push	bc
	ld	b,W_COLS	;Anzahl Bytes pro Zeile
sp_loop2:
	ld	c,(hl)
	ld	a,(de)
	ld	(hl),a
	ld	a,c
	ld	(de),a		;Bytes tauschen
	inc	hl
	inc	de
	djnz	sp_loop2	;noch nicht alle Bytes?

	ld	bc,OFFS-W_COLS
	add	hl,bc
	ex	de,hl
	add	hl,bc
	ex	de,hl		;Zeiger erhhen

	pop	bc
	djnz	sp_loop1	;noch nicht alle Zeilen?

	ret
;----------------------------------------------------------------
puts:
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	(scr_offs),de	;Offset holen
ps_loop:
	ld	a,(hl)
	push	hl
	call	putchar
	pop	hl		;Zeichen ausgeben

	bit	6,(hl)
	inc	hl
	jr	z,ps_loop	;noch nicht alle Zeichen?

	ret
;----------------------------------------------------------------
putchar:
	ld	hl,(scr_base)
	ld	de,(scr_offs)
	add	hl,de
	ex	de,hl		;Zeiger in Bildspeicher

	inc	hl
	ld	(scr_offs),hl	;neuer Offset

	ld	l,a
	add	hl,hl
	add	hl,hl
	ld	h,0
	add	hl,hl
	ld	bc,font
	add	hl,bc		;Zeiger in Font

	rla
	sbc	a,a
	ld	c,a		;Invertierung bei Bedarf

	ld	b,8
pc_loop:
	ld	a,(hl)
	xor	c
	ld	(de),a
	inc	hl
	ld	a,e
	add	a,OFFS
	ld	e,a
	jr	nc,pc_next
	inc	d
pc_next:
	djnz	pc_loop		;noch nicht ganzes Zeichen?

	ret

;= Bildschirm ===================================================

sync:
	nop
	nop
	ld	b,5
sy_delay1:
	djnz	sy_delay1	;Verzgerung fr eine Zeile

	ld	hl,keys
	ld	b,11111110B
	ld	c,KEYB
	ld	d,11111111B
sy_keyloop:
	in	a,(c)
	or	11100000B
	ld	(hl),a		;halbe Tastenzeile lesen

	and	d
	ld	d,a		;Summe mitfhren

	inc	hl
	rlc	b
	jr	c,sy_keyloop	;noch nicht alle Zeilen?

	ld	(hl),d		;Summe auch merken

	ld	b,59
	ld	b,59
sy_delay2:
	djnz	sy_delay2	;VSync ist sechs Zeilen lang

	out	(HSYNC_ON),a	;wieder Videopegel

	ld	a,MARGIN_TOP
	call	rstcal		;zum Hauptprogramm

	call	show		;Bild ausgeben

	ld	a,MARGIN_BOTTOM
	call	rstcal		;zum Hauptprogramm

	jr	sync		;und von vorn...
;----------------------------------------------------------------
show:
	ld	b,3
	ld	b,3
sh_delay:
	djnz	sh_delay	;Abstand von linken Rand

	ld	h,b
	ld	l,b
	add	hl,sp		;"echten" Stack merken
	ld	(line_stack+(1+LINES*2)*2),hl

	ld	sp,line_stack
	scf
	ret			;zum Dummycode

show_end:
	pop	hl
	ld	sp,hl		;wieder "echten" Stack setzen
	ret

;= Dummycode fr eine Bildzeile =================================

line:
	pop	hl	;nchste Bildadresse holen
	ld	a,h
	ld	i,a
	ld	a,l
	ld	r,a	;"Videozeiger" darauf richten
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop		;COLUMNS-mal "nichts tun"
	ret	c	;nchste Zeile / Bildende (C immer 1!)

LINE_LEN	equ	$-line

;= Bild =========================================================

packed:				;wird manuell angehngt

;= RAM ==========================================================

		org	4000H

keys:		ds	9		;eingelesene Tasten

dummy:		ds	LINE_LEN	;Dummycode

scr_base:	ds	2		;Startadresse Textschirm
scr_offs:	ds	2		;Offset in Textschirm

mode_act:	ds	1		;aktueller Modus

squixx:		ds	3*2*N_SQUIXX
speed:		ds	4		;fr Grafikeffekt

		ds	100		;Stack, reicht...
stack:

video_mem	equ	8000H-LINES*OFFS
		if	$ > video_mem
		error	"Zuviel RAM-Bedarf"
		endif

line_stack	equ	0A000H-(1+LINES*2)*2-2	;Dummystack

;= Ende =========================================================
	end
