;****************************************************************
;*  zx97mngr.asm
;*			Bootmanager fr ZX97
;*			====================
;*
;*  08.02.99	Bodo Wenzel	V1.0, Erstellung
;****************************************************************

	title	"ZX97 Manager V1.0"

;= I/O ==========================================================

POKE7		equ	0007H	;Adresse fr FPGA-Modus

NMI_OFF		equ	0FDH	;per OUT
NMI_ON		equ	0FEH	;per OUT
KEYB		equ	0FEH	;per IN
HSYNC_ON	equ	0FFH	;per OUT

;= Konstanten ===================================================

MODE_DEFAULT	equ	01111011B
			;7	RESET
			;6	0/1 = RAM/ROM in 0-8K
			;5	PAGE1
			;4	PAGE0 Seite fr 0-8K
			;3	0/1 = invers/normal
			;2	0/1 = HRG/Text in 8-16K
			;1	RAM1
			;0	RAM0 Modus fr RAM

;	ACHTUNG! Da die Bilderzeugung extrem vom durchlaufenen
;	Maschinencode abhngt, knnen diese Konstanten nicht
;	ohne weiteres gendert werden! Sie dienen nur der
;	besseren Lesbarkeit.

LINES		equ	240	;Anzahl Zeilen
COLUMNS		equ	320/8	;Anzahl Spalten [Bytes]
OFFS		equ	512/8	;nchstgrere 2er-Potenz
LINES_TOP	equ	3	;Leerzeilen oben
MARGIN_TOP	equ	8*LINES_TOP-1-1
MARGIN_BOTTOM	equ	312-(1+1+6+MARGIN_TOP+1+1+LINES)
				;sorry, keine 60 Hz mglich

;= Start (Reset) ================================================

	org	0000H
reset:
	out	(NMI_OFF),a
	di			;schnell stoppen

	ld	hl,line
	ld	de,dummy
	ld	bc,LINE_LEN
	ldir			;Dummycode ins RAM kopieren

	ld	sp,line_stack+(1+LINES*2)*2
	ld	hl,show_end
	push	hl
	ld	a,LINES
	ld	bc,dummy+8000H
	ld	hl,video_mem+LINES*OFFS
	ld	de,-OFFS
ls_loop:
	add	hl,de
	push	hl
	push	bc
	dec	a
	jr	nz,ls_loop	;Stack fr Bildausgabe setzen

	ld	sp,stack	;dies ist der "echte" Stack

	ld	ix,sync
	out	(NMI_ON),a	;Bildausgabe starten

	jr	unpack		;Bild entpacken

;= Dummycode fr eine Bildzeile =================================

line:
	pop	hl	;nchste Bildadresse holen
	ld	a,h
	ld	i,a
	ld	a,l
	ld	r,a	;"Videozeiger" darauf richten
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop		;COLUMNS-mal "nichts tun"
	ret	c	;nchste Zeile / Bildende (C immer 1!)

LINE_LEN	equ	$-line

;= NMI fr Zeilenzhlung ========================================

	if	$ > 0066H
	error	"Zuviel Code vor NMI"
	endif

	org	0066H
nmi:
	ex	af,af'
	dec	a
	jr	z,brkcal	;alle Leerzeilen bearbeitet ?
	ex	af,af'
	ret

brkcal:
	ex	af,af'
	push	af
	push	bc
	push	de
	push	hl		;Register retten

	halt			;synchronisieren mit HSync
	out	(NMI_OFF),a

	jp	(ix)		;abwechselnd "sync" und "show"
;----------------------------------------------------------------
rstcal:
	pop	ix		;Rcksprung merken

	ex	af,af'
	out	(NMI_ON),a	;NMIs zur Zeilenzhlung

	pop	hl
	pop	de
	pop	bc
	pop	af		;Register holen
	ret

;= Bild entpacken ===============================================

unpack:
	ld	hl,packed
	ld	de,video_mem+LINES*(OFFS-COLUMNS)
	ld	b,1		;Auspacken vorbereiten
	push	bc
unp_loop:
	pop	bc

	bit	7,d
	jr	nz,expand	;fertig mit Auspacken?

	djnz	unp_goon	;noch Bits im Prefixbyte?

	ld	c,(hl)		;nchstes Prefixbyte
	inc	hl
	ld	b,8		;Bitzhler zurcksetzen

unp_goon:
	rl	c
	push	bc
	jr	c,unp_repl	;Wiederholung?

	ldi			;Byte direkt kopieren
	jr	unp_loop

unp_repl:
	ld	c,(hl)
	inc	hl
	ld	a,(hl)
	and	0fh
	ld	b,a
	rld
	inc	hl		;Anzahl und Offset holen

	push	hl
	ld	hl,-3
	sbc	hl,bc
	add	hl,de
	add	a,3
	ld	c,a
	ld	b,0
	ldir
	pop	hl		;Wiederholung kopieren
	jr	unp_loop

expand:
	ld	hl,video_mem+LINES*(OFFS-COLUMNS)
	ld	de,video_mem
	ld	a,LINES		;Umspeicherung vorbereiten
exp_loop:
	ld	bc,COLUMNS
	ldir
	ex	de,hl
	ld	bc,OFFS-COLUMNS
	add	hl,bc
	ex	de,hl
	dec	a
	jr	nz,exp_loop	;noch nicht alle Zeilen?
;----------------------------------------------------------------

;	ACHTUNG, hier sind Konstanten vom Bild abhngig!

	ld	a,MODE_DEFAULT
	ld	(mode_act),a	;Standard-Modus vorgeben

	ld	de,video_mem+141*OFFS+30
	ld	c,00000001B	;aktuellen "Zeiger" vorbereiten

main_loop:
	exx
	ld	a,(mode_act)
	ld	hl,video_mem+141*OFFS+30+OFFS
	ld	de,-OFFS
	ld	b,8		;Modusanzeige vorbereiten

ml_showloop:
	ld	c,10110001B
	rra
	jr	nc,ml_showgoon
	ld	c,10001101B
ml_showgoon:
	ld	(hl),c
	add	hl,de
	ld	(hl),c
	add	hl,de
	ld	(hl),c
	add	hl,de
	add	hl,de
	add	hl,de
	add	hl,de
	djnz	ml_showloop	;Modus"schalter" anzeigen
	exx

	ld	a,(de)
	or	07EH
	ld	(de),a		;aktuelles Bit kennzeichnen

ml_waitkey:
	ld	hl,keys+8
ml_wait_off:
	ld	a,(hl)
	inc	a
	jr	nz,ml_wait_off	;auf Loslassen warten
ml_wait_on:
	ld	a,(hl)
	inc	a
	jr	z,ml_wait_on	;auf Tastendruck waretn

	dec	hl
	dec	hl
	bit	0,(hl)
	jr	z,ml_newline	;"newline" ?
	dec	hl
	dec	hl
	bit	3,(hl)
	jr	z,ml_up		;"cursor up"?
	bit	4,(hl)
	jr	z,ml_down	;"cursor down"?
	jr	ml_waitkey	;Rest ignorieren

ml_up:
	bit	7,c
	jr	nz,ml_waitkey	;noch nicht ganz oben?

	rlc	c
	ld	hl,-6*OFFS
	add	hl,de
	ex	de,hl		;nchstes Bit
	jr	main_loop
ml_down:
	bit	0,c
	jr	nz,ml_waitkey	;noch nicht ganz unten?

	rrc	c
	ld	hl,6*OFFS
	add	hl,de
	ex	de,hl		;nchstes Bit
	jr	main_loop

ml_newline:
	ld	hl,mode_act
	ld	a,(hl)
	xor	c
	ld	(hl),a		;aktuelles Bit wechseln

	bit	7,(hl)
	jr	z,main_loop	;noch kein RESET?

	halt
	out	(NMI_OFF),a	;Bildausgabe stoppen

	ld	(POKE7),a	;neues System starten...
				;(kommt bestimmt nicht weiter!)

;= Bildschirm ===================================================

sync:
	inc	hl
	inc	hl
	inc	hl
	ld	b,5
sy_delay1:
	djnz	sy_delay1	;Verzgerung fr eine Zeile

	ld	hl,keys
	ld	b,11111110B
	ld	c,KEYB
	ld	d,11111111B
sy_keyloop:
	in	a,(c)
	or	11100000B
	ld	(hl),a		;halbe Tastenzeile lesen

	and	d
	ld	d,a		;Summe mitfhren

	inc	hl
	rlc	b
	jr	c,sy_keyloop	;noch nicht alle Zeilen?

	ld	(hl),d		;Summe auch merken

	ld	b,59
	ld	b,59
sy_delay2:
	djnz	sy_delay2	;VSync ist sechs Zeilen lang

	out	(HSYNC_ON),a	;wieder Videopegel

	ld	a,MARGIN_TOP
	call	rstcal		;zum Hauptprogramm

	call	show		;Bild ausgeben

	ld	a,MARGIN_BOTTOM
	call	rstcal		;zum Hauptprogramm

	jr	sync		;und von vorn...
;----------------------------------------------------------------
show:
	nop
	ld	b,4
sh_delay:
	djnz	sh_delay	;Abstand von linken Rand

	ld	h,b
	ld	l,b
	add	hl,sp		;"echten" Stack merken
	ld	(line_stack+(1+LINES*2)*2),hl

	ld	sp,line_stack
	scf
	ret			;zum Dummycode

show_end:
	pop	hl
	ld	sp,hl		;wieder "echten" Stack setzen
	ret

;= Bild =========================================================

packed:				;wird manuell angehngt

;= RAM ==========================================================

		org	4000H

mode_act:	ds	1		;aktueller Modus
keys:		ds	9		;eingelesene Tasten

dummy:		ds	LINE_LEN	;Dummycode

		ds	100		;Stack, reicht...
stack:

video_mem	equ	8000H-LINES*OFFS
		if	$ > video_mem
		error	"Zuviel RAM-Bedarf"
		endif

line_stack	equ	0A000H-(1+LINES*2)*2-2	;Dummystack

;= Ende =========================================================
	end
